export function selectSource(room: Room): Id<Source> {
  console.log('Select source for room', room.name);
  const creepsInRoom = room.find(FIND_MY_CREEPS);
  const sourcesInRoom = room.find(FIND_SOURCES);
  const rates: Weights = {};
  for (let source of sourcesInRoom) {
    const creepCount = creepsInRoom.filter((c) => c.memory.sourceId === source.id).length;
    const weight = getSourceWeight(source);
    rates[source.id] = creepCount / weight;
  }
  console.log('Sources', JSON.stringify(rates));
  return getKeyWithMinWeight(rates) as Id<Source>;
}

function getSourceWeight(source: Source): number {
  const pos = source.pos;
  const terrains = source.room.lookForAtArea(LOOK_TERRAIN, pos.y - 1, pos.x - 1, pos.y + 1, pos.x + 1, true);
  const walked = terrains.filter((t) => t.terrain !== 'wall');
  return Math.max(walked.length - 1, 1);
}

interface Weights {
  [key: string]: number;
}

function getKeyWithMinWeight(weights: Weights): string {
  let keys = Object.keys(weights);
  return keys.reduce((key, v) => (weights[v] < weights[key] ? v : key));
}

export function harvestEnergyFromTombstone(creep: Creep) {
  const tombstones = creep.pos.findInRange(FIND_TOMBSTONES, 1, {
    filter: (t) => t.store[RESOURCE_ENERGY] > 0
  });
  for (let tomb of tombstones) {
    const res = creep.withdraw(tomb, RESOURCE_ENERGY);
    console.log('Withdraw', res);
  }
}
