import { harvestEnergyFromTombstone, selectSource } from './common';

const TARGET_TYPES = [STRUCTURE_EXTENSION, STRUCTURE_SPAWN, STRUCTURE_TOWER];

enum Action {
  Harvest = 'harvest',
  Charge = 'charge'
}

export function runAsHarvester(creep: Creep) {
  const memory = creep.memory as CreepMemory;

  if (memory.action === undefined) {
    memory.action = Action.Harvest;
  }

  if (!memory.sourceId) {
    memory.sourceId = selectSource(creep.room);
  }

  switch (memory.action) {
    case Action.Charge:
      if (creep.store[RESOURCE_ENERGY] === 0) {
        memory.action = Action.Harvest;
      }
      break;
    case Action.Harvest:
      if (creep.store.getFreeCapacity() === 0) {
        memory.action = Action.Charge;
      }
      break;
  }

  harvestEnergyFromTombstone(creep);

  switch (memory.action) {
    case Action.Harvest:
      const source = Game.getObjectById(memory.sourceId);
      if (source && creep.harvest(source) === ERR_NOT_IN_RANGE) {
        creep.say('🔄 harvest');
        creep.moveTo(source, { reusePath: 2, visualizePathStyle: { stroke: '#ffaa00' } });
      }
      break;
    case Action.Charge:
      const target = findTarget(creep);
      if (target) {
        moveEnergyToTarget(creep, target);
      } else {
        moveToSpawn(creep);
      }
      break;
  }
}

function findTarget(creep: Creep): Structure | undefined {
  for (let type of TARGET_TYPES) {
    const target = creep.pos.findClosestByPath(FIND_STRUCTURES, {
      filter: (structure) => {
        return structure.structureType === type && structure.store.getFreeCapacity(RESOURCE_ENERGY) > 0;
      }
    });
    if (target) {
      return target;
    }
  }
  return undefined;
}

function moveEnergyToTarget(creep: Creep, target: Structure) {
  if (creep.transfer(target, RESOURCE_ENERGY) === ERR_NOT_IN_RANGE) {
    creep.say('🚛 transfer');
    creep.moveTo(target, { reusePath: 2, visualizePathStyle: { stroke: '#ffffff' } });
  }
}

function moveToSpawn(creep: Creep) {
  const spawns = creep.room.find(FIND_MY_SPAWNS);
  if (spawns.length > 0) {
    creep.say('to spawn');
    creep.moveTo(spawns[0]);
  }
}
