import { harvestEnergyFromTombstone, selectSource } from './common';

interface UpgraderMemory extends CreepMemory {
  upgrading: boolean | undefined;
}

export function runAsUpgrader(creep: Creep) {
  const memory = creep.memory as UpgraderMemory;

  if (memory.upgrading && creep.store[RESOURCE_ENERGY] === 0) {
    memory.upgrading = false;
  }

  if (!memory.upgrading && creep.store.getFreeCapacity() === 0) {
    memory.upgrading = true;
  }

  if (!memory.sourceId) {
    memory.sourceId = selectSource(creep.room);
  }

  harvestEnergyFromTombstone(creep);

  if (memory.upgrading) {
    if (creep.room.controller && creep.upgradeController(creep.room.controller) == ERR_NOT_IN_RANGE) {
      creep.say('⚡ upgrade');
      creep.moveTo(creep.room.controller, { visualizePathStyle: { stroke: '#ffffff' } });
    }
  } else {
    const source = Game.getObjectById(memory.sourceId);
    if (source && creep.harvest(source) == ERR_NOT_IN_RANGE) {
      creep.say('🔄 harvest');
      creep.moveTo(source, { visualizePathStyle: { stroke: '#ffaa00' } });
    }
  }
}
